function ClassE3dviewer(D, db, stress)
% ClassE3dviewer ([0.3, 0.4, 0.5, 0.6, 0.7], [v4h1-std' 'v4h1-inv'], [true, false])
% D = 0.3, 0.4, 0.5, 0.6 or 0.7
% db is either 'v4h1-std' or 'v4h1-inv'
% stress is true for stress computing

if nargin < 1
    D = 0.5; end

if nargin < 2
    db = 'v4h1-std'; end

if nargin < 3
    stress = false; end

%% importa file e filtra database

FILEoptimal = sprintf('DCoptimal-%s-0%d.csv', db, round(10*D));
FILErelaxed = sprintf('DCrelaxed-%s-0%d.csv', db, round(10*D));
DBoptimal = dlmread(FILEoptimal,',',1,0);
DBrelaxed = dlmread(FILErelaxed,',',1,0);

% create optimal good and failed DB (only [x, y] for speed)
filter1 = and(-2.8-1e-4 <= DBoptimal(:,1), DBoptimal(:,1) <= 2.8+1e-4);
filter2 = and(-2.8-1e-4 <= DBoptimal(:,2), DBoptimal(:,2) <= 2.8+1e-4);
filter17 = DBoptimal(:,17) <= 1e-3;
xy_o = DBoptimal(logical(filter1.*filter2.*filter17),1:2);
%DBoptfailed = DBoptimal(~filter17);
    
% create optimal good and failed DB (only [x, y] for speed)
filter1 = and(-2.8-1e-4 <= DBrelaxed(:,1), DBrelaxed(:,1) <= 2.8+1e-4);
filter2 = and(-2.8-1e-4 <= DBrelaxed(:,2), DBrelaxed(:,2) <= 2.8+1e-4);
filter18 = DBrelaxed(:,18) <= 1e-3;
xy_r = DBrelaxed(logical(filter1.*filter2.*filter18),1:2);
%DBrelfailed = DBrelaxed(~filter18);

xysize_o = size(xy_o,1);
xysize_r = size(xy_r,1);
%xysize = 30;
    
%% crea plot 3D di qI qR qM iinv0 irec0

scale = 0.025;
xyq_o = round(xy_o./scale); % switching to integer
xyq_r = round(xy_r./scale); % switching to integer

xx = (min([xyq_o(:,1); xyq_r(:,1)]):max([xyq_o(:,1); xyq_r(:,1)]))*scale;
yy = (min([xyq_o(:,2); xyq_r(:,2)]):max([xyq_o(:,2); xyq_r(:,2)]))*scale;

zz_o_qI = NaN(size(xx,2),size(yy,2));
zz_r_qI = NaN(size(xx,2),size(yy,2));
zz_o_qR = NaN(size(xx,2),size(yy,2));
zz_r_qR = NaN(size(xx,2),size(yy,2));
zz_o_qM = NaN(size(xx,2),size(yy,2));
zz_r_qM = NaN(size(xx,2),size(yy,2));
zz_o_iinv0 = NaN(size(xx,2),size(yy,2));
zz_r_iinv0 = NaN(size(xx,2),size(yy,2));
zz_o_irec0 = NaN(size(xx,2),size(yy,2));
zz_r_irec0 = NaN(size(xx,2),size(yy,2));

for jj = 1: xysize_o
    kI = xy_o(jj,1); kR = xy_o(jj,2);
    [theline, dist] = line_extract_nearest_v4b(DBoptimal,[kI,kR]);
    xi = round((kI-xx(1))/scale)+1; yi = round((kR-yy(1))/scale)+1;
    zz_o_qI(xi,yi) = theline(3);
    zz_o_qR(xi,yi) = theline(4);
    zz_o_qM(xi,yi) = theline(5);
    zz_o_iinv0(xi,yi) = theline(6);
    zz_o_irec0(xi,yi) = theline(7);
end

for jj = 1: xysize_r
    kI = xy_r(jj,1); kR = xy_r(jj,2);
    [theline, dist] = line_extract_nearest_v4b(DBrelaxed,[kI,kR]);
    xi = round((kI-xx(1))/scale)+1; yi = round((kR-yy(1))/scale)+1;
    zz_r_qI(xi,yi) = theline(3);
    zz_r_qR(xi,yi) = theline(4);
    zz_r_qM(xi,yi) = theline(5);
    zz_r_iinv0(xi,yi) = theline(6);
    zz_r_irec0(xi,yi) = theline(7);
end

% non ho capito perch� devo usare Z(y,x)???

myFigHandle = findobj('Tag','EvalParamqI');
if isvalid(myFigHandle); close(myFigHandle); end
figure('Tag', 'EvalParamqI');
mesh (xx, yy, zz_o_qI'); hold on;
colormap winter; freezeColors; hold on;
mesh (xx, yy, zz_r_qI'); 
colormap autumn; freezeColors; hold off;
title(sprintf('%s-%g',db,D));
xlabel('k_I'); 
ylabel('k_R');
zlabel('qI');

myFigHandle = findobj('Tag','EvalParamqR');
if isvalid(myFigHandle); close(myFigHandle); end
figure('Tag', 'EvalParamqR');
mesh (xx, yy, zz_o_qR'); hold on;
colormap winter; freezeColors; hold on;
mesh (xx, yy, zz_r_qR'); 
colormap autumn; freezeColors; hold off;
title(sprintf('%s-%g',db,D));
xlabel('k_I'); 
ylabel('k_R');
zlabel('qR');

myFigHandle = findobj('Tag','EvalParamqM');
if isvalid(myFigHandle); close(myFigHandle); end
figure('Tag', 'EvalParamqM');
mesh (xx, yy, zz_o_qM'); hold on;
colormap winter; freezeColors; hold on;
mesh (xx, yy, zz_r_qM'); 
colormap autumn; freezeColors; hold off;
title(sprintf('%s-%g',db,D));
xlabel('k_I'); 
ylabel('k_R');
zlabel('qM');

myFigHandle = findobj('Tag','EvalParamiinv0');
if isvalid(myFigHandle); close(myFigHandle); end
figure('Tag', 'EvalParamiinv0');
mesh (xx, yy, zz_o_iinv0'); 
colormap winter; freezeColors; hold on;
mesh (xx, yy, zz_r_iinv0'); 
colormap autumn; freezeColors; hold off;
title(sprintf('%s-%g',db,D));
xlabel('k_I'); 
ylabel('k_R');
zlabel('iinv0');

myFigHandle = findobj('Tag','EvalParamirec0');
if isvalid(myFigHandle); close(myFigHandle); end
figure('Tag', 'EvalParamirec0');
mesh (xx, yy, zz_o_irec0'); 
colormap winter; freezeColors; hold on;
mesh (xx, yy, zz_r_irec0'); 
colormap autumn; freezeColors; hold off;
title(sprintf('%s-%g',db,D));
xlabel('k_I'); 
ylabel('k_R');
zlabel('irec0');

if stress
    fprintf('Computing stress values, please wait...');
    
    %% Mi tocca fare la matrice...
    zz_o_vdspk = NaN(size(xx,2),size(yy,2));
    zz_r_vdspk = NaN(size(xx,2),size(yy,2));
    zz_o_vkapk = NaN(size(xx,2),size(yy,2));
    zz_r_vkapk = NaN(size(xx,2),size(yy,2));
    zz_o_iinv0rms = NaN(size(xx,2),size(yy,2));
    zz_r_iinv0rms = NaN(size(xx,2),size(yy,2));
    zz_o_irec0rms = NaN(size(xx,2),size(yy,2));
    zz_r_irec0rms = NaN(size(xx,2),size(yy,2));

    %% simulate all points to get the stress values (takes between 1 and 2 minutes)

    %parameters definitions--> ideal case
    QI=inf;
    QR=inf;
    QM=inf;
    QCinv = inf;
    QCrec = inf; 
    vdON = 0;                    
    vbON = 0; 
    gINV = inf;
    gdsON = inf;
    gbON = inf;
    gCM = inf;
    gdON = inf;
    gREC = inf;
    

    tic;
    for jj = 1:xysize_o

        %repear for every kI,kR
        kI = xy_o(jj,1);
        kR = xy_o(jj,2);


        % parameters extraction from Database
        [theline, dist] = line_extract_nearest_v4b(DBoptimal,[kI,kR]);

        qI = theline(3);
        qR = theline(4);
        qM = theline(5);
        iinv0 = theline(6); 
        irec0 = theline(7);
        vka0 = theline(8);
        res_optimal=theline(17);


        %Debug -- show/calculate the waveform once optimization is complete

        [R, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
                vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);


        T = 2*pi*(0:255)'/256;
        Y = zeros(256,4);
        for ii=1:256
            Y(ii,:) = fZ(T(ii));
        end
            
%            if (true)
%            plot([T;T+2*pi], [Y;Y])
%            legend('iinv(\theta)','irec(\theta)','vDS(\theta)','vKA(\theta)')  
%            end
            
        % Evaluate stress  
        %vds_peak(jj)=max(Y(:,3)) ;
        %vka_peak(jj)=max(Y(:,4)) ;
        %iinv_rms(jj)=sqrt(norm(Y(:,1))^2/256);
        %irec_rms(jj)=sqrt(norm(Y(:,2))^2/256);    
        xi = round((kI-xx(1))/scale)+1; yi = round((kR-yy(1))/scale)+1;
        zz_o_vdspk(xi,yi) = max(Y(:,3));
        zz_o_vkapk(xi,yi) = max(Y(:,4));
        zz_o_iinv0rms(xi,yi) = sqrt(norm(Y(:,1))^2/256);
        zz_o_irec0rms(xi,yi) = sqrt(norm(Y(:,2))^2/256);
           
    end

        for jj = 1:xysize_r

        %repear for every kI,kR
        kI = xy_r(jj,1);
        kR = xy_r(jj,2);


        % parameters extraction from Database
        [theline, dist] = line_extract_nearest_v4b(DBrelaxed,[kI,kR]);

        qI = theline(3);
        qR = theline(4);
        qM = theline(5);
        iinv0 = theline(6); 
        irec0 = theline(7);
        vka0 = theline(8);
        res_optimal=theline(17);


        %Debug -- show/calculate the waveform once optimization is complete

        [R, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
                vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);


        T = 2*pi*(0:255)'/256;
        Y = zeros(256,4);
        for ii=1:256
            Y(ii,:) = fZ(T(ii));
        end
            
%            if (true)
%            plot([T;T+2*pi], [Y;Y])
%            legend('iinv(\theta)','irec(\theta)','vDS(\theta)','vKA(\theta)')  
%            end
            
        % Evaluate stress  
        %vds_peak(jj)=max(Y(:,3)) ;
        %vka_peak(jj)=max(Y(:,4)) ;
        %iinv_rms(jj)=sqrt(norm(Y(:,1))^2/256);
        %irec_rms(jj)=sqrt(norm(Y(:,2))^2/256);    
        xi = round((kI-xx(1))/scale)+1; yi = round((kR-yy(1))/scale)+1;
        zz_r_vdspk(xi,yi) = max(Y(:,3));
        zz_r_vkapk(xi,yi) = max(Y(:,4));
        zz_r_iinv0rms(xi,yi) = sqrt(norm(Y(:,1))^2/256);
        zz_r_irec0rms(xi,yi) = sqrt(norm(Y(:,2))^2/256);
           
        end
    
    toc

    myFigHandle = findobj('Tag','EvalStressvdspk');
    if isvalid(myFigHandle); close(myFigHandle); end
    figure('Tag', 'EvalStressvdspk');
    mesh (xx, yy, zz_o_vdspk'); hold on;
    colormap winter; freezeColors; hold on;
    mesh (xx, yy, zz_r_vdspk'); 
    colormap autumn; freezeColors; hold off;
    title(sprintf('%s-%g',db,D));
    xlabel('k_I'); 
    ylabel('k_R');
    zlabel('vdspk');

    myFigHandle = findobj('Tag','EvalStressvkapk');
    if isvalid(myFigHandle); close(myFigHandle); end
    figure('Tag', 'EvalStressvkapk');
    mesh (xx, yy, zz_o_vkapk'); hold on;
    colormap winter; freezeColors; hold on;
    mesh (xx, yy, zz_r_vkapk'); 
    colormap autumn; freezeColors; hold off;
    title(sprintf('%s-%g',db,D));
    xlabel('k_I'); 
    ylabel('k_R');
    zlabel('vkapk');

    myFigHandle = findobj('Tag','Evalstressiinv0rms');
    if isvalid(myFigHandle); close(myFigHandle); end
    figure('Tag', 'Evalstressiinv0rms');
    mesh (xx, yy, zz_o_iinv0rms'); 
    colormap winter; freezeColors; hold on;
    mesh (xx, yy, zz_r_iinv0rms'); 
    colormap autumn; freezeColors; hold off;
    title(sprintf('%s-%g',db,D));
    xlabel('k_I'); 
    ylabel('k_R');
    zlabel('iinv0rms');

    myFigHandle = findobj('Tag','Evalstressirec0rms');
    if isvalid(myFigHandle); close(myFigHandle); end
    figure('Tag', 'Evalstressirec0rms');
    mesh (xx, yy, zz_o_irec0rms'); 
    colormap winter; freezeColors; hold on;
    mesh (xx, yy, zz_r_irec0rms'); 
    colormap autumn; freezeColors; hold off;
    title(sprintf('%s-%g',db,D));
    xlabel('k_I'); 
    ylabel('k_R');
    zlabel('irec0rms');


end % if stress



end

function freezeColors(varargin)
% freezeColors  Lock colors of plot, enabling multiple colormaps per figure. (v2.3)
%
%   Problem: There is only one colormap per figure. This function provides
%       an easy solution when plots using different colomaps are desired 
%       in the same figure.
%
%   freezeColors freezes the colors of graphics objects in the current axis so 
%       that subsequent changes to the colormap (or caxis) will not change the
%       colors of these objects. freezeColors works on any graphics object 
%       with CData in indexed-color mode: surfaces, images, scattergroups, 
%       bargroups, patches, etc. It works by converting CData to true-color rgb
%       based on the colormap active at the time freezeColors is called.
%
%   The original indexed color data is saved, and can be restored using
%       unfreezeColors, making the plot once again subject to the colormap and
%       caxis.
%
%
%   Usage:
%       freezeColors        applies to all objects in current axis (gca),
%       freezeColors(axh)   same, but works on axis axh.
%
%   Example:
%       subplot(2,1,1); imagesc(X); colormap hot; freezeColors
%       subplot(2,1,2); imagesc(Y); colormap hsv; freezeColors etc...
%
%       Note: colorbars must also be frozen. Due to Matlab 'improvements' this can
%				no longer be done with freezeColors. Instead, please
%				use the function CBFREEZE by Carlos Adrian Vargas Aguilera
%				that can be downloaded from the MATLAB File Exchange
%				(http://www.mathworks.com/matlabcentral/fileexchange/24371)
%
%       h=colorbar; cbfreeze(h), or simply cbfreeze(colorbar)
%
%       For additional examples, see test/test_main.m
%
%   Side effect on render mode: freezeColors does not work with the painters
%       renderer, because Matlab doesn't support rgb color data in
%       painters mode. If the current renderer is painters, freezeColors
%       changes it to zbuffer. This may have unexpected effects on other aspects
%	      of your plots.
%
%       See also unfreezeColors, freezeColors_pub.html, cbfreeze.
%
%
%   John Iversen (iversen@nsi.edu) 3/23/05
%
%   Changes:
%   JRI (iversen@nsi.edu) 4/19/06   Correctly handles scaled integer cdata
%   JRI 9/1/06   should now handle all objects with cdata: images, surfaces, 
%                scatterplots. (v 2.1)
%   JRI 11/11/06 Preserves NaN colors. Hidden option (v 2.2, not uploaded)
%   JRI 3/17/07  Preserve caxis after freezing--maintains colorbar scale (v 2.3)
%   JRI 4/12/07  Check for painters mode as Matlab doesn't support rgb in it.
%   JRI 4/9/08   Fix preserving caxis for objects within hggroups (e.g. contourf)
%   JRI 4/7/10   Change documentation for colorbars
% Hidden option for NaN colors:
%   Missing data are often represented by NaN in the indexed color
%   data, which renders transparently. This transparency will be preserved
%   when freezing colors. If instead you wish such gaps to be filled with 
%   a real color, add 'nancolor',[r g b] to the end of the arguments. E.g. 
%   freezeColors('nancolor',[r g b]) or freezeColors(axh,'nancolor',[r g b]),
%   where [r g b] is a color vector. This works on images & pcolor, but not on
%   surfaces.
%   Thanks to Fabiano Busdraghi and Jody Klymak for the suggestions. Bugfixes 
%   attributed in the code.
% Free for all uses, but please retain the following:
%   Original Author:
%   John Iversen, 2005-10
%   john_iversen@post.harvard.edu
appdatacode = 'JRI__freezeColorsData';
[h, nancolor] = checkArgs(varargin);
%gather all children with scaled or indexed CData
cdatah = getCDataHandles(h);
%current colormap
cmap = colormap;
nColors = size(cmap,1);
cax = caxis;
% convert object color indexes into colormap to true-color data using 
%  current colormap
for hh = cdatah',
    g = get(hh);
    
    %preserve parent axis clim
    parentAx = getParentAxes(hh);
    originalClim = get(parentAx, 'clim');    
   
    %   Note: Special handling of patches: For some reason, setting
    %   cdata on patches created by bar() yields an error,
    %   so instead we'll set facevertexcdata instead for patches.
    if ~strcmp(g.Type,'patch'),
        cdata = g.CData;
    else
        cdata = g.FaceVertexCData; 
    end
    
    %get cdata mapping (most objects (except scattergroup) have it)
    if isfield(g,'CDataMapping'),
        scalemode = g.CDataMapping;
    else
        scalemode = 'scaled';
    end
    
    %save original indexed data for use with unfreezeColors
    siz = size(cdata);
    setappdata(hh, appdatacode, {cdata scalemode});
    %convert cdata to indexes into colormap
    if strcmp(scalemode,'scaled'),
        %4/19/06 JRI, Accommodate scaled display of integer cdata:
        %       in MATLAB, uint * double = uint, so must coerce cdata to double
        %       Thanks to O Yamashita for pointing this need out
        idx = ceil( (double(cdata) - cax(1)) / (cax(2)-cax(1)) * nColors);
    else %direct mapping
        idx = cdata;
        %10/8/09 in case direct data is non-int (e.g. image;freezeColors)
        % (Floor mimics how matlab converts data into colormap index.)
        % Thanks to D Armyr for the catch
        idx = floor(idx);
    end
    
    %clamp to [1, nColors]
    idx(idx<1) = 1;
    idx(idx>nColors) = nColors;
    %handle nans in idx
    nanmask = isnan(idx);
    idx(nanmask)=1; %temporarily replace w/ a valid colormap index
    %make true-color data--using current colormap
    realcolor = zeros(siz);
    for i = 1:3,
        c = cmap(idx,i);
        c = reshape(c,siz);
        c(nanmask) = nancolor(i); %restore Nan (or nancolor if specified)
        realcolor(:,:,i) = c;
    end
    
    %apply new true-color color data
    
    %true-color is not supported in painters renderer, so switch out of that
    if strcmp(get(gcf,'renderer'), 'painters'),
        set(gcf,'renderer','zbuffer');
    end
    
    %replace original CData with true-color data
    if ~strcmp(g.Type,'patch'),
        set(hh,'CData',realcolor);
    else
        set(hh,'faceVertexCData',permute(realcolor,[1 3 2]))
    end
    
    %restore clim (so colorbar will show correct limits)
    if ~isempty(parentAx),
        set(parentAx,'clim',originalClim)
    end
    
end %loop on indexed-color objects

end
% ============================================================================ %
% Local functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% getCDataHandles -- get handles of all descendents with indexed CData
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function hout = getCDataHandles(h)
% getCDataHandles  Find all objects with indexed CData
%recursively descend object tree, finding objects with indexed CData
% An exception: don't include children of objects that themselves have CData:
%   for example, scattergroups are non-standard hggroups, with CData. Changing
%   such a group's CData automatically changes the CData of its children, 
%   (as well as the children's handles), so there's no need to act on them.
error(nargchk(1,1,nargin,'struct'))
hout = [];
if isempty(h),return;end
ch = get(h,'children');
for hh = ch'
    g = get(hh);
    if isfield(g,'CData'),     %does object have CData?
        %is it indexed/scaled?
        if ~isempty(g.CData) && isnumeric(g.CData) && size(g.CData,3)==1, 
            hout = [hout; hh]; %#ok<AGROW> %yes, add to list
        end
    else %no CData, see if object has any interesting children
            hout = [hout; getCDataHandles(hh)]; %#ok<AGROW>
    end
end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% getParentAxes -- return handle of axes object to which a given object belongs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function hAx = getParentAxes(h)
% getParentAxes  Return enclosing axes of a given object (could be self)
error(nargchk(1,1,nargin,'struct'))
%object itself may be an axis
if strcmp(get(h,'type'),'axes'),
    hAx = h;
    return
end
parent = get(h,'parent');
if (strcmp(get(parent,'type'), 'axes')),
    hAx = parent;
else
    hAx = getParentAxes(parent);
end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% checkArgs -- Validate input arguments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [h, nancolor] = checkArgs(args)
% checkArgs  Validate input arguments to freezeColors
nargs = length(args);
error(nargchk(0,3,nargs,'struct'))
%grab handle from first argument if we have an odd number of arguments
if mod(nargs,2),
    h = args{1};
    if ~ishandle(h),
        error('JRI:freezeColors:checkArgs:invalidHandle',...
            'The first argument must be a valid graphics handle (to an axis)')
    end
    % 4/2010 check if object to be frozen is a colorbar
    if strcmp(get(h,'Tag'),'Colorbar'),
      if ~exist('cbfreeze.m'),
        warning('JRI:freezeColors:checkArgs:cannotFreezeColorbar',...
            ['You seem to be attempting to freeze a colorbar. This no longer'...
            'works. Please read the help for freezeColors for the solution.'])
      else
        cbfreeze(h);
        return
      end
    end
    args{1} = [];
    nargs = nargs-1;
else
    h = gca;
end
%set nancolor if that option was specified
nancolor = [nan nan nan];
if nargs == 2,
    if strcmpi(args{end-1},'nancolor'),
        nancolor = args{end};
        if ~all(size(nancolor)==[1 3]),
            error('JRI:freezeColors:checkArgs:badColorArgument',...
                'nancolor must be [r g b] vector');
        end
        nancolor(nancolor>1) = 1; nancolor(nancolor<0) = 0;
    else
        error('JRI:freezeColors:checkArgs:unrecognizedOption',...
            'Unrecognized option (%s). Only ''nancolor'' is valid.',args{end-1})
    end
end

end

function varargout = line_extract_nearest_v4b(data, kk, reslim)
% find nearest line - if not existing, update kI, kR and residuals
% additional third arguments is [res5, res4]: minimum residual to be looked
% for

    % newline = line_extract_nearest_v4a(database, [kI, kR])
    %  -- or --
    % [newline, distance] = line_extract_nearest_v4a(database, [kI, kR])
    %  -- or --
    % [newline, distance, npoints] = line_extract_nearest_v4a(database, [kI, kR])

    % per trovare (kI, kR) calcolo distanza dai punti

    % otional third argument
    if nargin >= 3
        % pre-filter data keeping only good results
        filter17 = data(:,17) <= reslim(1);
        filter18 = data(:,18) <= reslim(2);
        data = data(logical(filter17.*filter18),:);
    end    
    % NOTE: pnorm not available in all systems... redo with norm
    %pp = vecnorm(data(:,1:2)-kk, 2, 2);
    data = data(randperm(size(data,1)),:); % random shuffle
    pp = (data(:,1)-kk(1)).^2+(data(:,2)-kk(2)).^2;


    [~, indx] = sort(pp);
    newline = data(indx(1),:);

    if norm(newline(1:2)-kk) > 1e-4 % 0 with tolerance
        % clear residuals and update kI, kR if not existing
        newline(1:2) = kk;
        newline(9:16) = zeros(1,8);
        newline(17:18) = [1., 1.];
        newline(19:32) = -ones(1,14);
    end

    switch nargout
        case {0, 1}
            varargout = {newline};
        case 2
            varargout = {newline, sqrt(pp(indx(1)))};
        case 3
            % 
            varargout = {newline, sqrt(pp(indx(1))), sum(abs(pp-pp(indx(1)))<1e-6)};
        otherwise
            warning('Wrong output argument number, no output set.')
    end

    
end