function ClassEviewer(type, myStruct, varargin)
% TO DO:
% check max & min value of text (edit boxes)
% check value of kI and kR when panning
% check value of kI and kR when zooming
% add secondary plot with values of qI qR etc (immediate check for problems)
% position the secondary windows
% IN PROGRESS -- on changing duty cycle, do not change xrange yrange kI kR & checked box
% DONE -- enable 2h
% DONE -- force a square form factor while zooming
% DONE -- on resize, change the plot size according to the available space
% DONE -- set the step of the slider to 0.025 / 10% dynamically according to the zoom level and the shift  
% DONE -- adjust max & min slider value according to zoom level & shift
% DONE -- identify the children by tag and not by the order
% DONE -- close child window on exit
% DONE -- select the duty cycle from drop menu
% DONE -- unify main and callback program (check for varargin arguments)
% DONE -- redraw the same top window if existing
% DONE -- auto naming with mfilename
% DONE -- close the secondary plot if not necessary
% DONE -- reset main figure instead of closing and recreating

    if nargin == 0
        % If no arguments, then create the main window
        ClassEviewerCreateMainplot(false);
    else
        % otherwise, call the callback function
        ClassEviewerCallback(type, myStruct, varargin{:});
    end
end

function ClassEviewerCallback(type, myStruct, varargin)
    
    % Otherwise, we are in the callback function
    switch type
        case 'debug'
            mainfigHandle = findobj('Tag','MainClassEviewerWindow');
            debugfigHandle = findobj('Tag','DebugClassEviewerWindow');
            % Look for kI, kR and the database
            kI =  get(findobj(mainfigHandle,'Tag','kIslider'), 'Value');
            kR =  get(findobj(mainfigHandle,'Tag','kRslider'), 'Value');
            % The duty-cycle should be taken from the popup menu
            objectHandle = findobj(groot,'Tag','duty-cycle');
            db = objectHandle.Value;
            DBoptimal = myStruct.DB(db).optimal;
            lineopt = ClassEviewer_line_extract_nearest_v4b(DBoptimal, [kI,kR]);
            linestr = sprintf('%.12g, ',lineopt);
            linestr = linestr(1:end-2); % remove the last ', '
            if lineopt(17) < myStruct.goodlim(2)
                tmp = get(findobj(debugfigHandle,'Tag','thetext'), 'String');
                if isempty(tmp)
                    set(findobj(debugfigHandle,'Tag','thetext'), 'String', linestr);
                else
                    tmp2=tmp(1,:);
                    for jj=2:size(tmp,1)
                        tmp2 = sprintf('%s\n%s',tmp2,tmp(jj,:));
                    end
                    set(findobj(debugfigHandle,'Tag','thetext'), 'String', sprintf('%s\n%s',linestr,tmp2));
                end
            end
            return
        case 'resize'
            thefigHandle = varargin{1};
            % Position is [x y width height]
            %width = thefigHandle.Position(3);
            %heigth = thefigHandle.Position(4);
            %fprintf("size: %g, %g\n", width, heigth);
            ClassEviewerSetmainplotdensity(myStruct, thefigHandle);
            return;
        case 'kIplot'
            % toggle the evolution windows
            objectHandle = varargin{1};
            thefigHandle = objectHandle.Parent;
            kI =  get(findobj(thefigHandle,'Tag','kIslider'), 'Value');
            ClassEviewerReplotsidefigurekI(myStruct, kI);
            return
        case 'kRplot'
            % toggle the evolution windows
            objectHandle = varargin{1};
            thefigHandle = objectHandle.Parent;
            kR =  get(findobj(thefigHandle,'Tag','kRslider'), 'Value');
            ClassEviewerReplotsidefigurekR(myStruct, kR);
            return
        case '1hplot'
            % toggle the evolution windows
            objectHandle = varargin{1};
            thefigHandle = objectHandle.Parent;
            kI =  get(findobj(thefigHandle,'Tag','kIslider'), 'Value');
            kR =  get(findobj(thefigHandle,'Tag','kRslider'), 'Value');
            replotsecondaryevo(myStruct, kI, kR);
            return
        case 'dchoice'
            objectHandle = varargin{1};
            db = objectHandle.Value;
            % close the fig and redraw from scratch
            ClassEreplotprimary(myStruct,db);
            return
        case 'closemain' 
            thefigHandle = varargin{1};
            sidefigHandle = findobj(groot,'Tag','SideFigure-1h');
            close(sidefigHandle);
            sidefigHandle = findobj(groot,'Tag','SideFigure-kI');
            close(sidefigHandle);
            sidefigHandle = findobj(groot,'Tag','SideFigure-kR');
            close(sidefigHandle);            
%            sidefigHandle = findobj(groot,'Tag','SideWindow-2h');
%            close(sidefigHandle);
            delete(thefigHandle); % close current figure
        case 'zoom'
            thefigHandle = varargin{1};
            axesHandle = varargin{2}.Axes;
            step = myStruct.step;
            xlim = step*round(axesHandle.XLim/step);
            ylim = step*round(axesHandle.YLim/step);
            % shortstep is set to 0.025, largestep to 0.25 regardless of the zoom size
            % steps are to be passed as a percentage value of the range
            xsteps = step/diff(xlim) * [1 10];
            ysteps = step/diff(ylim) * [1 10];
            set(axesHandle, 'XLim', xlim);
            set(axesHandle, 'YLim', ylim);
            set(findobj(axesHandle,'Tag','kRline'), 'XData', xlim, 'LineWidth', .5*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','kIline'), 'YData', ylim, 'LineWidth', .5*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','optimal-good'), 'LineWidth', .02*2.5/diff(xlim), 'MarkerSize', 2*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','optimal-fail'), 'LineWidth', .5*2.5/diff(xlim), 'MarkerSize', 3*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','relaxed-good'), 'LineWidth', .02*2.5/diff(xlim), 'MarkerSize', 2*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','relaxed-fail'), 'LineWidth', .5*2.5/diff(xlim), 'MarkerSize', 3*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','existenceline-neg'), 'LineWidth', 1.0*2.5/diff(xlim));
            set(findobj(axesHandle,'Tag','existenceline-pos'), 'LineWidth', 1.0*2.5/diff(xlim));
            %set(findobj(axesHandle,'Tag','optimal-h2'), 'LineWidth', 0.1*2.5/diff(xlim), 'MarkerSize', 1.66*2.5/diff(xlim));
            % set new kI and kR if out of range
            kI =  get(findobj(thefigHandle,'Tag','kIslider'), 'Value');
            kR =  get(findobj(thefigHandle,'Tag','kRslider'), 'Value');
            if kI < xlim(1); kI=xlim(1); end
            if kR < ylim(1); kR=ylim(1); end
            if kI > xlim(2); kI=xlim(2); end
            if kR > ylim(2); kR=ylim(2); end
            set(findobj(thefigHandle,'Tag','kIslider'), 'Min', xlim(1), 'Max', xlim(2), 'Value', kI, 'SliderStep', xsteps);
            set(findobj(thefigHandle,'Tag','kRslider'), 'Min', ylim(1), 'Max', ylim(2), 'Value', kR, 'SliderStep', ysteps);
            ClassEviewerSetmainplotdensity(myStruct, thefigHandle);
        case 'pan'
            thefigHandle = varargin{1};
            axesHandle = varargin{2}.Axes;
            step = myStruct.step;
            xlim = step*round(axesHandle.XLim/step);
            ylim = step*round(axesHandle.YLim/step);
            set(axesHandle, 'XLim', xlim);
            set(axesHandle, 'YLim', ylim);
            set(findobj(axesHandle,'Tag','kRline'), 'XData', xlim);
            set(findobj(axesHandle,'Tag','kIline'), 'YData', ylim);
            % set new kI and kR if out of range
            kI =  get(findobj(thefigHandle,'Tag','kIslider'), 'Value');
            kR =  get(findobj(thefigHandle,'Tag','kRslider'), 'Value');
            if kI < xlim(1); kI=xlim(1); end
            if kR < ylim(1); kR=ylim(1); end
            if kI > xlim(2); kI=xlim(2); end
            if kR > ylim(2); kR=ylim(2); end
            set(findobj(thefigHandle,'Tag','kIslider'), 'Min', xlim(1), 'Max', xlim(2), 'Value', kI);
            set(findobj(thefigHandle,'Tag','kRslider'), 'Min', ylim(1), 'Max', ylim(2), 'Value', kR);
        case 'kIslider'
            objectHandle = varargin{1};
            %eventData = varargin{2}; % not useful here -- .EventName='Action' & .Source=objectHandle
            thefigHandle = objectHandle.Parent;
            step = myStruct.step;
            kI = step*round((objectHandle.Value/step)); % ensure to be on the grid
        case 'kRslider'
            objectHandle = varargin{1};
            %eventData = varargin{2}; % not useful here -- .EventName='Action' & .Source=objectHandle
            thefigHandle = objectHandle.Parent;
            step = myStruct.step;
            kR = step*round((objectHandle.Value/step)); % ensure to be on the grid
        case 'kIedit'
            objectHandle = varargin{1};
            %eventData = varargin{2}; % not useful here -- .EventName='Action' & .Source=objectHandle
            thefigHandle = objectHandle.Parent;
            step = myStruct.step;
            kI = step*round((str2double(objectHandle.String)/step)); % ensure to be on the grid
        case 'kRedit'
            objectHandle = varargin{1};
            %eventData = varargin{2}; % not useful here -- .EventName='Action' & .Source=objectHandle
            thefigHandle = objectHandle.Parent;
            step = myStruct.step;
            kR = step*round((str2double(objectHandle.String)/step)); % ensure to be on the grid
        case 'mouseclick'
            objectHandle = varargin{1};
            %eventData = varargin{2}; % not useful here -- .EventName='Action' & .Source=objectHandle
            axesHandle  = get(objectHandle,'Parent');
            thefigHandle = axesHandle.Parent;
            coordinates = get(axesHandle,'CurrentPoint'); 
            %coordinates = coordinates(1,1:2);
            %message     = sprintf('x: %.1f , y: %.1f',coordinates (1) ,coordinates (2));
            %helpdlg(message);
            step = myStruct.step;
            kI = step*round(coordinates(1,1)/step);
            kR = step*round(coordinates(1,2)/step);            
        otherwise       
            return;
    end

    % structures children of the main figure:
    %   'ExistencePlotAxes', 'kIslider', 'kRslider', 'kIedit', 'kRedit'
    %
    % structures children of 'ExistencePlotAxes'
    %   'kIline', 'kRline'
    

    % update everything
    if exist('kI', 'var') % kI has been changed
        set(findobj(thefigHandle,'Tag','kIslider'), 'Value', kI);
        set(findobj(thefigHandle,'Tag','kIedit'), 'String', sprintf('%g',kI));
        axesHandle = findobj(thefigHandle,'Tag','ExistencePlotAxes');
        set(findobj(axesHandle,'Tag','kIline'), 'XData', [kI kI]);
        ClassEviewerReplotsidefigurekI(myStruct, kI);
    end
    if exist('kR', 'var') % kR has been changed
        set(findobj(thefigHandle,'Tag','kRslider'), 'Value', kR);
        set(findobj(thefigHandle,'Tag','kRedit'), 'String', sprintf('%g',kR));
        axesHandle = findobj(thefigHandle,'Tag','ExistencePlotAxes');
        set(findobj(axesHandle,'Tag','kRline'), 'YData', [kR kR]);
        ClassEviewerReplotsidefigurekR(myStruct, kR);
    end
    if exist('kI', 'var') || exist('kR', 'var') % kI or kR has been changed
        kI =  get(findobj(thefigHandle,'Tag','kIslider'), 'Value');
        kR =  get(findobj(thefigHandle,'Tag','kRslider'), 'Value');
        replotsecondaryevo(myStruct, kI, kR);
    end
    
end
    
function ClassEviewerReplotsidefigurekI(myStruct, kI)
% plot the secondary plot (parameters for kI fixed)


    % need to look if a figure with tag 'SideFigure-kI' is open
    sidefigHandle = findobj(groot,'Tag','SideFigure-kI');
    
    % check if SideFigure-kI has to be plotted or not
    objectHandle = findobj(groot,'Tag','secondarykIbox');
    boxflag = objectHandle.Value;
    if(not(boxflag))
        % figure not to be shown
        close(sidefigHandle);
        return
    end
    
    if isvalid(sidefigHandle)
        % figure is existing, do nothing just clear the figure
        % (warning - also the tag is cleared, need to reset!)
        clf(sidefigHandle,'reset')
        set(sidefigHandle, 'Tag', 'SideFigure-kI');
    else
        % re-create the figure
        sidefigHandle = figure('Tag', 'SideFigure-kI');
    end
    
    % The duty-cycle should be taken from the popup menu
    objectHandle = findobj(groot,'Tag','duty-cycle');
    db = objectHandle.Value;
    DBoptimal = myStruct.DB(db).optimal;
    DBrelaxed = myStruct.DB(db).relaxed;
    %theta2 = myStruct.theta2(db);
    
    axesHandle = axes('Parent',sidefigHandle,'Tag','kIparameterPlot');

    hold(axesHandle,'on');

    % This code really needs to be improved...
    if myStruct.DB(db).optimalisloaded
        filter1 = and( kI - 1e-4 <= DBoptimal(:,1), DBoptimal(:,1) <= kI + 1e-4); % [sum(filter17) sum(not(filter17))]
        filter17 = and( myStruct.goodlim(1) <= DBoptimal(:,17), DBoptimal(:,17) <= myStruct.goodlim(2)); % [sum(filter17) sum(not(filter17))]
        mypointsopt1h = DBoptimal(logical(filter1.*filter17),1:8);
        [~, indexes] = sort(mypointsopt1h(:,2));
        mypointsopt1h = mypointsopt1h(indexes,:);
    else
        mypointsopt1h = [];
    end
    
    if ~isempty(mypointsopt1h) % if size(mypointsopt1h,1) > 0 
        % plot only if optimal points exist
        plot(axesHandle, mypointsopt1h(:,3:8), mypointsopt1h(:,2));
    end

    if myStruct.DB(db).relaxedisloaded
        filter1 = and( kI - 1e-4 <= DBrelaxed(:,1), DBrelaxed(:,1) <= kI + 1e-4); % [sum(filter17) sum(not(filter17))]
        filter18 = and( myStruct.goodlim(1) <= DBrelaxed(:,18), DBrelaxed(:,18) <= myStruct.goodlim(2)); % [sum(filter17) sum(not(filter17))]
        mypointsrel1h = DBrelaxed(logical(filter1.*filter18),1:8);
        [~, indexes] = sort(mypointsrel1h(:,2));
        mypointsrel1h = mypointsrel1h(indexes,:);
    else
        mypointsrel1h = [];
    end
    
    if ~isempty(mypointsrel1h)
        % plot only if relaxed points exist
        set(axesHandle,'ColorOrderIndex',1); % reset color scheme
        plot(axesHandle, mypointsrel1h(:,3:8), mypointsrel1h(:,2),'--');
    end
    
    % three different legends according to the existance of optimal and relaxed 
    if ~isempty(mypointsopt1h) && ~isempty(mypointsrel1h)
        % both db have points 
        legend(axesHandle, {'qI (opt.)', 'qR (opt.)', 'qM (opt.)', 'iinv0 (opt.)', 'irec0 (opt.)', 'vka0 (opt.)', ...
                            'qI (rel.)', 'qR (rel.)', 'qM (rel.)', 'iinv0 (rel.)', 'irec0 (rel.)', 'vka0 (rel.)'});
    end
    if ~isempty(mypointsopt1h) && isempty(mypointsrel1h)
        % only optimal 
        legend(axesHandle, {'qI (opt.)', 'qR (opt.)', 'qM (opt.)', 'iinv0 (opt.)', 'irec0 (opt.)', 'vka0 (opt.)'});
    end
    if isempty(mypointsopt1h) && ~isempty(mypointsrel1h)
        % only relaxed 
        legend(axesHandle, {'qI (rel.)', 'qR (rel.)', 'qM (rel.)', 'iinv0 (rel.)', 'irec0 (rel.)', 'vka0 (rel.)'});
    end
    
    hold(axesHandle,'off');
end

function ClassEviewerReplotsidefigurekR(myStruct, kR)
% plot the secondary plot (parameters for kR fixed)


    % need to look if a figure with tag 'SideFigure-kR' is open
    sidefigHandle = findobj(groot,'Tag','SideFigure-kR');
    
    % check if SideFigure-kR has to be plotted or not
    objectHandle = findobj(groot,'Tag','secondarykRbox');
    boxflag = objectHandle.Value;
    if(not(boxflag))
        % figure not to be shown
        close(sidefigHandle);
        return
    end
    
    if isvalid(sidefigHandle)
        % figure is existing, do nothing just clear the figure
        % (warning - also the tag is cleared, need to reset!)
        clf(sidefigHandle,'reset')
        set(sidefigHandle, 'Tag', 'SideFigure-kR');
    else
        % re-create the figure
        sidefigHandle = figure('Tag', 'SideFigure-kR');
    end
    
    % The duty-cycle should be taken from the popup menu
    objectHandle = findobj(groot,'Tag','duty-cycle');
    db = objectHandle.Value;
    DBoptimal = myStruct.DB(db).optimal;
    DBrelaxed = myStruct.DB(db).relaxed;
    %theta2 = myStruct.theta2(db);
    
    axesHandle = axes('Parent',sidefigHandle,'Tag','kIparameterPlot');

    hold(axesHandle,'on');
    
    if myStruct.DB(db).optimalisloaded
        filter2 = and( kR - 1e-4 <= DBoptimal(:,2), DBoptimal(:,2) <= kR + 1e-4); % [sum(filter17) sum(not(filter17))]
        filter17 = and( myStruct.goodlim(1) <= DBoptimal(:,17), DBoptimal(:,17) <= myStruct.goodlim(2)); % [sum(filter17) sum(not(filter17))]
        mypointsopt1h = DBoptimal(logical(filter2.*filter17),1:8);
        [~, indexes] = sort(mypointsopt1h(:,1));
        mypointsopt1h = mypointsopt1h(indexes,:);
    else
        mypointsopt1h = [];
    end
    
    if ~isempty(mypointsopt1h) % if size(mypointsopt1h,1) > 0 
        % plot only if optimal points exist
        plot(axesHandle, mypointsopt1h(:,1), mypointsopt1h(:,3:8));
    end

    if myStruct.DB(db).relaxedisloaded
        filter2 = and( kR - 1e-4 <= DBrelaxed(:,2), DBrelaxed(:,2) <= kR + 1e-4); % [sum(filter17) sum(not(filter17))]
        filter18 = and( myStruct.goodlim(1) <= DBrelaxed(:,18), DBrelaxed(:,18) <= myStruct.goodlim(2)); % [sum(filter17) sum(not(filter17))]
        mypointsrel1h = DBrelaxed(logical(filter2.*filter18),1:8);
        [~, indexes] = sort(mypointsrel1h(:,1));
        mypointsrel1h = mypointsrel1h(indexes,:);
    else
        mypointsrel1h = [];
    end
    
    if ~isempty(mypointsrel1h)
        % plot only if relaxed points exist
        set(axesHandle,'ColorOrderIndex',1); % reset color scheme
        plot(axesHandle, mypointsrel1h(:,1), mypointsrel1h(:,3:8),'--'); 
    end
    
    % three different legends according to the existance of optimal and relaxed 
    if ~isempty(mypointsopt1h) && ~isempty(mypointsrel1h)
        % both db have points 
        legend(axesHandle, {'qI (opt.)', 'qR (opt.)', 'qM (opt.)', 'iinv0 (opt.)', 'irec0 (opt.)', 'vka0 (opt.)', ...
                            'qI (rel.)', 'qR (rel.)', 'qM (rel.)', 'iinv0 (rel.)', 'irec0 (rel.)', 'vka0 (rel.)'});
    end
    if ~isempty(mypointsopt1h) && isempty(mypointsrel1h)
        % only optimal 
        legend(axesHandle, {'qI (opt.)', 'qR (opt.)', 'qM (opt.)', 'iinv0 (opt.)', 'irec0 (opt.)', 'vka0 (opt.)'});
    end
    if isempty(mypointsopt1h) && ~isempty(mypointsrel1h)
        % only relaxed 
        legend(axesHandle, {'qI (rel.)', 'qR (rel.)', 'qM (rel.)', 'iinv0 (rel.)', 'irec0 (rel.)', 'vka0 (rel.)'});
    end
    
    hold(axesHandle,'off');

end


function replotsecondaryevo(myStruct, kI, kR)
% plot the secondary plot (evolution of the converter)

    % The duty-cycle should be taken from the popup menu
    objectHandle = findobj(groot,'Tag','duty-cycle');
    db = objectHandle.Value;
    DBoptimal1h = myStruct.DB(db).optimal;
    DBrelaxed1h = myStruct.DB(db).relaxed;
    %theta2 = myStruct.theta2(db);
    theta2 = myStruct.DB(db).theta2;
    
    % need to look if a figure with tag 'sideFigure-1h' is open
    sidefigHandle = findobj(groot,'Tag','SideFigure-1h');

    % check if secondary-evo has to be plotted or not
    objectHandle = findobj(groot,'Tag','secondaryevobox');
    boxflag = objectHandle.Value;
    if(not(boxflag))
        % figure not to be shown
        close(sidefigHandle);
        return
    end
        
    % try the optimal DB
    line1h = ClassEviewer_line_extract_nearest_v4b(DBoptimal1h, [kI,kR]);
    dbstr = 'Optimal';
    if line1h(17) > myStruct.goodlim(2)
        % Then, try to the relaxed db
        line1h = ClassEviewer_line_extract_nearest_v4b(DBrelaxed1h, [kI,kR]);
        dbstr = 'Relaxed';
        if line1h(18) > myStruct.goodlim(2)
            % Nothing good in optimal nor in relaxed; close
            close(sidefigHandle);
            return
        end
    end
    
    if isvalid(sidefigHandle)
        % figure is existing, do nothing just clear the figure
        % (warning - also the tag is cleared, need to reset!)
        clf(sidefigHandle,'reset')
        set(sidefigHandle, 'Tag', 'SideFigure-1h');
    else
        % re-create the figure
        sidefigHandle = figure('Tag', 'SideFigure-1h');
    end
        
    axesHandle = axes('Parent',sidefigHandle,'Tag','EvolutionPlot');
    %TODO check theta2 with line1h(15)             
    %            theta2 = line1h(15);

    [~, fZ, thetas, string] = solver4b (line1h(6), line1h(7), line1h(8), theta2, line1h(3), line1h(4), line1h(5), line1h(1), line1h(2), ...
        0., 0., inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf);

    T = 2*pi*(0:255)'/255;
    Y = zeros(length(T),4);
    for ii=1:length(T)
        Y(ii,:) = fZ(T(ii));
    end
    plot(axesHandle, [T; T+2*pi ], [Y; Y], '-')
    title(axesHandle, {[sprintf('%s,',dbstr), sprintf(' D=%s',myStruct.DB(db).string), sprintf(', sequence: %s', string)],...
                           sprintf('k_I=%g, k_R=%g, q_I=%g, q_R=%g q_M=%g',kI,kR,line1h(3), line1h(4), line1h(5)),...
                       ['0', sprintf(' -> %g',thetas(1:end))]});
    legend(axesHandle,'i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)');

    

%     % need to look if a figure with tag 'sideFigure-2h' is open
%         sidefigHandle = findobj(groot,'Tag','SideWindow-2h');
%         if isvalid(sidefigHandle)
%             % figure is existing, do nothing just clear the figure
%             % (warning - also the tag is cleared, need to reset!)
%             clf(sidefigHandle,'reset')
%             set(sidefigHandle, 'Tag', 'SideWindow-2h');
%         else
%             % re-create the figure
%             sidefigHandle = figure('Tag', 'SideWindow-2h');
%         end
        
%         if line2h(17) < 0.001
%    
% % todo: code may be more intelligent, do not need to redraw all       
%     axesHandle = axes('Parent',sidefigHandle,'Tag','EvolutionPlot');
%     
%             theta2 = line2h(15);
% 
%       [R, fZ, theta] = solver4b (line2h(6), line2h(7), line2h(8), theta2, line2h(3), line2h(4), line2h(5), line2h(1), line2h(2), ...
%         0., 0., inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf);
%       T = 2*pi*(0:255)'/255;
%         Y = zeros(length(T),4);
%         for ii=1:length(T)
%             Y(ii,:) = fZ(T(ii));
%         end
%         %ax = gca; ax.ColorOrderIndex = 1; % reuse the same colors
%         axesHandle.ColorOrderIndex = 1; % reuse the same colors
%         plot(axesHandle, [T; T+2*pi ], [Y; Y],'-')
%         title(axesHandle, sprintf('(2h) kI=%g, kR=%g, qI=%g, qR=%g qM=%g',kI,kR,line2h(3), line2h(4), line2h(5)));
%         legend(axesHandle,'i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)');
% 
% 
%     end
end

function ClassEviewerCreateMainplot(openshowline)

% It would be nice to embed all data as a child of a figure handle; I am
% not able to do it for now, so I create my own structure with all data to
% be plotted

% Note: the structure is static, cannot be modified once this code is
% terminated (i.e., cannot be used to memorize data even if passed to child
% objects)
myStruct = struct('Content','ClassEData'); 

type_str = {'v4h1', 'v4h2', 'v4h3', 'v4h4', 'v4h5', 'v4h6'};
type_desc = {'', ' (h2)', ' (h3)', ' (h4)', ' (h5)', ' (h6)'};
D_str = {'01', '02', '03', '04', '05', '06', '07', '08', '09'};
D_value = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9];
D_desc = {'10%', '20%', '30%', '40%', '50%', '60%', '70%', '80%', '90%'};

% Load all databases and put into the struct -- should check for files
emptydb = struct('optimal', [], ...
                 'relaxed', [], ...
                 'theta2', 0, ...
                 'string' , '', ...
                 'optimalisloaded', false, ...
                 'relaxedisloaded', false);

myStruct.DB=repmat(emptydb,1,size(type_str,2)*size(D_str,2));

for jj = 1:size(type_str,2)
    for kk = 1:size(D_str,2)
        db_no = kk + (jj-1)*size(D_str,2); % used many times
        % this is the string to describe the db
        myStruct.DB(db_no).string = [D_desc{kk} type_desc{jj}];
        % this is the theta2
        myStruct.DB(db_no).theta2 = 2*pi*D_value(kk);
        % Look for the 4 files - starting from optimal, std
        filename = ['DCoptimal-' type_str{jj} '-std-' D_str{kk} '.csv'];
        if exist(filename, 'file')
            myStruct.DB(db_no).optimal = [myStruct.DB(db_no).optimal; dlmread(filename,',',1,0)];
            myStruct.DB(db_no).optimalisloaded = true;
            disp([filename ' loaded.']);
        end
        % now test the optimal, inv
        filename = ['DCoptimal-' type_str{jj} '-inv-' D_str{kk} '.csv'];
        if exist(filename, 'file')
            myStruct.DB(db_no).optimal = [myStruct.DB(db_no).optimal; dlmread(filename,',',1,0)];
            myStruct.DB(db_no).optimalisloaded = true;
            disp([filename ' loaded.']);
        end
        % finally, relaxed both std and inv
        filename = ['DCrelaxed-' type_str{jj} '-std-' D_str{kk} '.csv'];
        if exist(filename, 'file')
            myStruct.DB(db_no).relaxed = [myStruct.DB(db_no).relaxed; dlmread(filename,',',1,0)];
            myStruct.DB(db_no).relaxedisloaded = true;
            disp([filename ' loaded.']);
        end
        filename = ['DCrelaxed-' type_str{jj} '-inv-' D_str{kk} '.csv'];
        if exist(filename, 'file')
            myStruct.DB(db_no).relaxed = [myStruct.DB(db_no).relaxed; dlmread(filename,',',1,0)];
            myStruct.DB(db_no).relaxedisloaded = true;
            disp([filename ' loaded.']);
        end
    end
end

% now remove the empty dbs
dbtokeep = zeros(size(type_str,2)*size(D_str,2),1);
for db_no = 1:size(type_str,2)*size(D_str,2)
    dbtokeep(db_no) = or(myStruct.DB(db_no).optimalisloaded, myStruct.DB(db_no).relaxedisloaded);
end
myStruct.DB = myStruct.DB(logical(dbtokeep));


%myStruct.theta2 = 2*pi*[0.3, 0.4, 0.5, 0.6, 0.7];
%myStruct.dbstring = {'30%', '40%', '50%', '60%', '70%'};
%myStruct.dbdefault = 3;
myStruct.xrangedefault = [0., 2.8];
myStruct.yrangedefault = [0., 2.];
myStruct.goodlim = [0., 1e-3]; % limits for considering a point a good one
myStruct.kIdeafult = 1.0;
myStruct.kRdeafult = 1.0;

myStruct.step = .025;
myStruct.density = 1/170; % dots per unity

% mathematica colors:
% .368417, .506779, .709798
% .880722, .611041, .142051
% .560181, .691569, .194885
% .922526, .385626, .209179

ClassEreplotprimary(myStruct, 1);

if openshowline
    % for debugging purposes only
    ClassEreplotdebug(myStruct);
end

end

function ClassEreplotdebug(myStruct)

% To get the name of this function
viewervcallbackf = str2func(mfilename);

% Close the same figure if already present
debugfigHandle = findobj('Tag','DebugClassEviewerWindow');

if isvalid(debugfigHandle)
    close(debugfigHandle); end

debugfigHandle = figure('Tag', 'DebugClassEviewerWindow');

    uicontrol('Parent', debugfigHandle, 'style', 'pushbutton',...
            'Tag', 'thebutton', ... to be able to identify it
            'units', 'normalized', 'position', [0.4,0.01,0.2,0.06],...
            'Callback' , @(varargin) viewervcallbackf('debug', myStruct, varargin{:}),...
            'String','Copy this point');
         

    uicontrol( 'Parent', debugfigHandle, 'style', 'edit',...
            'Tag', 'thetext', ... to be able to identify it
            'Max', 2, ...
            'units', 'normalized', 'position', [0.01,0.1,0.99,.89],...
            'String', '' );

end


function ClassEreplotprimary(myStruct, db)

% To get the name of this function
viewervcallbackf = str2func(mfilename);

% Close the same figure if already present
mainfigHandle = findobj('Tag','MainClassEviewerWindow');

if isvalid(mainfigHandle)
    % figure is existing, do nothing just clear the figure
    axesHandle = findobj(mainfigHandle,'Tag','ExistencePlotAxes');
    % save xrange, yrange, kI, kR
    xrangedefault = axesHandle.XLim;
    yrangedefault = axesHandle.YLim;
    kI =  get(findobj(mainfigHandle,'Tag','kIslider'), 'Value');
    kR =  get(findobj(mainfigHandle,'Tag','kRslider'), 'Value');
    % (warning - also the tag is cleared, need to reset!)
    clf(mainfigHandle,'reset')
    set(mainfigHandle, 'Tag', 'MainClassEviewerWindow');
    % close child windows
    sidefigHandle = findobj(groot,'Tag','SideFigure-1h');
    close(sidefigHandle);
else
    % default values for xrange, yrange, kI, kR
    xrangedefault = myStruct.xrangedefault;
    yrangedefault = myStruct.yrangedefault;
    kI = myStruct.kIdeafult;
    kR = myStruct.kRdeafult;
    % re-create the figure
    mainfigHandle = figure('Tag', 'MainClassEviewerWindow');
end
    
%if isvalid(oldfigHandle)
%if not(isempty(oldfigHandle))
%    close(oldfigHandle);
%end
%    mainfigHandle = figure('Tag', 'MainClassEviewerWindow');

axesHandle = axes('Parent',mainfigHandle,'Tag','ExistencePlotAxes');

DBoptimal = myStruct.DB(db).optimal;
DBrelaxed = myStruct.DB(db).relaxed;

% used for the existence line kR*kI <= 1
kImax = 2.5; kImin = -2.5; 
kRmax = 2.5; kRmin = -2.5; 

if myStruct.DB(db).optimalisloaded
    filter17 = and( myStruct.goodlim(1) <= DBoptimal(:,17), DBoptimal(:,17) <= myStruct.goodlim(2)); % [sum(filter17) sum(not(filter17))]
    xyDBoptimalgood = DBoptimal(filter17,1:2);
    xyDBoptimalfail = DBoptimal(~filter17,1:2);
    kImax = max([kImax; xyDBoptimalgood(:,1)]);
    kImin = min([kImin; xyDBoptimalgood(:,1)]);
    kRmax = max([kRmax; xyDBoptimalgood(:,2)]);
    kRmin = min([kRmin; xyDBoptimalgood(:,2)]);
end

if myStruct.DB(db).relaxedisloaded
    filter18 = and( myStruct.goodlim(1) <= DBrelaxed(:,18), DBrelaxed(:,18) <= myStruct.goodlim(2)); % [sum(filter18) sum(not(filter18))]
    xyDBrelaxedgood = DBrelaxed(filter18,1:2);
    xyDBrelaxedfail = DBrelaxed(~filter18,1:2);
    kImax = max([kImax; xyDBrelaxedgood(:,1)]);
    kImin = min([kImin; xyDBrelaxedgood(:,1)]);
    kRmax = max([kRmax; xyDBrelaxedgood(:,2)]);
    kRmin = min([kRmin; xyDBrelaxedgood(:,2)]);
end

hold (axesHandle, 'on');

%plot existance line kR=1/kI
% from (kImin, 1/kImin) to (-1,-1)
x1 = kImin:myStruct.step:-1; y1=1./x1;
% from (-1,-1) to (1/kRmin, kRmin)
y2 = -1:-myStruct.step:kRmin; x2=1./y2;
plot(axesHandle, [x1 x2], [y1 y2], '--r', 'Tag','existenceline-neg', 'LineWidth', 1.0);
% from (1/kRmax, kRmax) to (1,1)
y3 = kRmax:-myStruct.step:1; x3=1./y3;
% from (-1,-1) to (kImax, 1/kImax)
x4 = 1:myStruct.step:kImax; y4=1./x4;
plot(axesHandle, [x3 x4], [y3 y4], '--r', 'Tag','existenceline-pos', 'LineWidth', 1.0);

if myStruct.DB(db).optimalisloaded
    % Plot optimal, good (o) and fail (+)
    poh1g = plot(axesHandle, xyDBoptimalgood(:,1),xyDBoptimalgood(:,2),'o',...
        'Tag','optimal-good',...
        'MarkerEdgeColor',[.368417, .506779, .709798],...
        'MarkerFaceColor',[.368417, .506779, .709798],... [.49 1 .63],...
        'LineWidth', 0.5,...
        'MarkerSize',2);
    plot(axesHandle, xyDBoptimalfail(:,1),xyDBoptimalfail(:,2),'+',...
        'Tag','optimal-fail',...
        'MarkerEdgeColor',[.880722, .611041, .142051],...
        'MarkerFaceColor',[.880722, .611041, .142051],... [.49 1 .63],...
        'LineWidth', .5,...
        'MarkerSize',3);
end

if myStruct.DB(db).relaxedisloaded
    % Plot relaxed, good (o) and fail (x)
    prh1g = plot(axesHandle, xyDBrelaxedgood(:,1),xyDBrelaxedgood(:,2),'o',...
        'Tag','relaxed-good',...
        'MarkerEdgeColor','k',...
        'MarkerFaceColor',[.560181, .691569, .194885],... [.79 .4 .63],...
        'LineWidth', 0.02,...
        'MarkerSize',2);
    plot(axesHandle, xyDBrelaxedfail(:,1),xyDBrelaxedfail(:,2),'x',...
        'Tag','relaxed-fail',...
        'MarkerEdgeColor',[.922526, .385626, .209179],...
        'MarkerFaceColor',[.922526, .385626, .209179],... [.49 1 .63],...
        'LineWidth', .5,...
        'MarkerSize',3);
end

plot(axesHandle, [kI, kI], yrangedefault ,'-k', 'Tag', 'kIline', 'LineWidth', .5);

plot(axesHandle, xrangedefault ,[kR, kR] ,'-k', 'Tag', 'kRline', 'LineWidth', .5);

axesHandle.XLim =xrangedefault;
axesHandle.YLim =yrangedefault;

hold (axesHandle, 'off');

% there is a problem above, cannot set the tag of axes (it will be lost if not
% set as the the last operation)
axesHandle.Tag = 'ExistencePlotAxes';
% axesHandle.Position; %  
% Default position is: 0.1300    0.1100    0.7750    0.8150

% default width (rrelative)
% x: 2.8*170/mainfigHandle.Position(3) = .85
% y: 2.0*170/mainfigHandle.Position(4) = .8095

axesx0 = .1;
axesy0 = .11;
axesx1 = axesx0+.85;
axesy1 = axesy0+.8095;
sliderheight = .04;
textheight = .04;
textwidth = .055;

axesHandle.Position = [axesx0, axesy0, axesx1-axesx0,axesy1-axesy0];

% Parent is used for being able to address myfigHandle and all its children
uicontrol(  'Parent', mainfigHandle, 'style', 'edit',...
            'Tag', 'kIedit', ... to be able to identify it
            'units', 'normalized', 'position', [axesx0-0.01-textwidth,0.01,textwidth,textheight],...
            'Callback' , @(varargin) viewervcallbackf('kIedit', myStruct, varargin{:}),...
            'String', sprintf('%g',kI) );
            
uicontrol( 'Parent', mainfigHandle, 'style', 'edit',...
            'Tag', 'kRedit', ... to be able to identify it
            'units', 'normalized', 'position', [0.01,axesy0-0.01-textheight,textwidth,textheight],...
            'Callback' , @(varargin) viewervcallbackf('kRedit', myStruct, varargin{:}),...
            'String', sprintf('%g',kR) );
        
uicontrol(  'Parent', mainfigHandle, 'Style', 'slider',...
            'Tag', 'kIslider', ... to be able to identify it
            'units', 'normalized','Position',[axesx0,0.01,axesx1-axesx0,sliderheight],... X, Y, DX, DY
            'SliderStep', [0.0100 0.1000], ...
            'Callback' , @(varargin) viewervcallbackf('kIslider', myStruct, varargin{:}),...
            'value', kI, 'min',0, 'max', 2.5 );
            
uicontrol( 'Parent', mainfigHandle, 'style', 'slider',...
            'Tag', 'kRslider', ... to be able to identify it
            'units', 'normalized', 'position', [0.01,axesy0,sliderheight*0.75,axesy1-axesy0],... X, Y, DX, DY
            'SliderStep', [0.0100 0.1000], ...
            'Callback' , @(varargin) viewervcallbackf('kRslider', myStruct, varargin{:}),...
            'value', kR, 'min', 0, 'max', 2.5);

uicontrol( 'Parent', mainfigHandle, 'style', 'popupmenu',...
            'Tag', 'duty-cycle', ... to be able to identify it
            'units', 'normalized', 'position', [axesx0,0.99-sliderheight,0.25,sliderheight],...
            'Callback' , @(varargin) viewervcallbackf('dchoice', myStruct, varargin{:}),...
            'String', {myStruct.DB(:).string}, 'Value', db );

uicontrol( 'Parent', mainfigHandle, 'style', 'checkbox',...
            'Tag', 'secondaryevobox', ... to be able to identify it
            'units', 'normalized', 'position', [0.4,0.99-sliderheight,0.25,sliderheight],...
            'Callback' , @(varargin) viewervcallbackf('1hplot', myStruct, varargin{:}),...
            'String', 'Show evolution' );
        
uicontrol( 'Parent', mainfigHandle, 'style', 'checkbox',...
            'Tag', 'secondarykIbox', ... to be able to identify it
            'units', 'normalized', 'position', [0.58,0.99-sliderheight,0.25,sliderheight],...
            'Callback' , @(varargin) viewervcallbackf('kIplot', myStruct, varargin{:}),...
            'String', 'k_I' );

uicontrol( 'Parent', mainfigHandle, 'style', 'checkbox',...
            'Tag', 'secondarykRbox', ... to be able to identify it
            'units', 'normalized', 'position', [0.65,0.99-sliderheight,0.25,sliderheight],...
            'Callback' , @(varargin) viewervcallbackf('kRplot', myStruct, varargin{:}),...
            'String', 'k_R' );
                
% position dovrebbe essere: X Y DX DY [0,0 � angolo in basso a sinistra]
%set(mysuperplot.ax,'ButtonDownFcn',@(es,ed) solutionsviewercallback('mouseclick', skI, skR, vkI, vkR, mysuperplot)); 
%mysuperplot.ax.HitTest = 'off';

% ButtonDownFcn on good plots, not on fail plots
if myStruct.DB(db).optimalisloaded
    set(poh1g, 'ButtonDownFcn', @(varargin) viewervcallbackf('mouseclick', myStruct, varargin{:}));
end
if myStruct.DB(db).relaxedisloaded
    set(prh1g, 'ButtonDownFcn', @(varargin) viewervcallbackf('mouseclick', myStruct, varargin{:}));
end

panHandle = pan(mainfigHandle);
panHandle.ActionPostCallback = @(varargin) viewervcallbackf('pan', myStruct, varargin{:});

zoomHandle = zoom(mainfigHandle);
zoomHandle.ActionPostCallback = @(varargin) viewervcallbackf('zoom', myStruct, varargin{:});

% dangerous to handle the closing of a figure
set(mainfigHandle, 'CloseRequestFcn', @(varargin) viewervcallbackf('closemain', myStruct, varargin{:}));
%    set(sidefigHandle, 'CloseRequestFcn', []);  % disable closing possibility

set(mainfigHandle, 'SizeChangedFcn', @(varargin) viewervcallbackf('resize', myStruct, varargin{:}));

%bl1 = uicontrol('Parent',mainfigHandle,'Style','text','Position',[50,54,23,23],...
%                'String','0','BackgroundColor',bgcolor);
%bl2 = uicontrol('Parent',mainfigHandle,'Style','text','Position',[500,54,23,23],...
%                'String','1','BackgroundColor',bgcolor);
%bl3 = uicontrol('Parent',mainfigHandle,'Style','text','Position',[240,25,100,23],...
%                'String','Damping Ratio','BackgroundColor',bgcolor);

% adjust axes, range, and markers and lines size for a constant density
ClassEviewerSetmainplotdensity(myStruct,mainfigHandle);

end

function ClassEviewerSetmainplotdensity(myStruct, mainfigHandle)

    axesHandle = findobj(mainfigHandle,'Tag','ExistencePlotAxes');

    % 1. look for the center of the axes to be preserved
    figcenter = [mean(axesHandle.XLim) mean(axesHandle.YLim)];
    
    % 2. look for the density on the two axes
    % width and heigth in pixel of the axes
    axeswidth = mainfigHandle.Position(3)*axesHandle.Position(3);
    axesheight = mainfigHandle.Position(4)*axesHandle.Position(4);
    xdensity = diff(axesHandle.XLim)/axeswidth;
    ydensity = diff(axesHandle.YLim)/axesheight;
    density = norm([xdensity ydensity])/sqrt(2);   
    % [1/xdensity 1/ydensity 1/density]

    % 3. compute the xscale and yscale according to the required density
    xscale = axeswidth * density;
    yscale = axesheight * density;
    xrange = myStruct.step*round((figcenter(1)+xscale*[-0.5,0.5])/myStruct.step);
    yrange = myStruct.step*round((figcenter(2)+yscale*[-0.5,0.5])/myStruct.step);
    
    ratio = myStruct.density/density;
    

    set(axesHandle, 'XLim', xrange);
    set(axesHandle, 'YLim', yrange);
    set(findobj(axesHandle,'Tag','kRline'), 'XData', xrange, 'LineWidth', .5*ratio);
    set(findobj(axesHandle,'Tag','kIline'), 'YData', yrange, 'LineWidth', .5*ratio);
    set(findobj(axesHandle,'Tag','optimal-good'), 'LineWidth', .02*ratio, 'MarkerSize', 2*ratio);
    set(findobj(axesHandle,'Tag','optimal-fail'), 'LineWidth', .5*ratio, 'MarkerSize', 3*ratio);
    set(findobj(axesHandle,'Tag','relaxed-good'), 'LineWidth', .02*ratio, 'MarkerSize', 2*ratio);
    set(findobj(axesHandle,'Tag','relaxed-fail'), 'LineWidth', .5*ratio, 'MarkerSize', 3*ratio);
    set(findobj(axesHandle,'Tag','existenceline-neg'), 'LineWidth', 1.0*ratio);
    set(findobj(axesHandle,'Tag','existenceline-pos'), 'LineWidth', 1.0*ratio);
    set(findobj(mainfigHandle,'Tag','kIslider'), 'Min', xrange(1), 'Max', xrange(2));
    set(findobj(mainfigHandle,'Tag','kRslider'), 'Min', yrange(1), 'Max', yrange(2));
    %set(findobj(axesHandle,'Tag','optimal-h2'), 'LineWidth', 0.1*2.5/diff(xlim), 'MarkerSize', 1.66*2.5/diff(xlim));
            
end

function varargout = ClassEviewer_line_extract_nearest_v4b(data, kk, reslim)
% find nearest line - if not existing, update kI, kR and residuals
% additional third arguments is [res5, res4]: minimum residual to be looked
% for

    % newline = line_extract_nearest_v4a(database, [kI, kR])
    %  -- or --
    % [newline, distance] = line_extract_nearest_v4a(database, [kI, kR])
    %  -- or --
    % [newline, distance, npoints] = line_extract_nearest_v4a(database, [kI, kR])

    % per trovare (kI, kR) calcolo distanza dai punti

    % otional third argument
    if nargin >= 3
        % pre-filter data keeping only good results
        filter17 = data(:,17) <= reslim(1);
        filter18 = data(:,18) <= reslim(2);
        data = data(logical(filter17.*filter18),:);
    end    
    % NOTE: pnorm not available in all systems... redo with norm
    %pp = vecnorm(data(:,1:2)-kk, 2, 2);
    data = data(randperm(size(data,1)),:); % random shuffle
    pp = (data(:,1)-kk(1)).^2+(data(:,2)-kk(2)).^2;


    [~, indx] = sort(pp);
    newline = data(indx(1),:);

    if norm(newline(1:2)-kk) > 1e-4 % 0 with tolerance
        % clear residuals and update kI, kR if not existing
        newline(1:2) = kk;
        newline(9:16) = zeros(1,8);
        newline(17:18) = [1., 1.];
        newline(19:32) = -ones(1,14);
    end

    switch nargout
        case {0, 1}
            varargout = {newline};
        case 2
            varargout = {newline, sqrt(pp(indx(1)))};
        case 3
            % 
            varargout = {newline, sqrt(pp(indx(1))), sum(abs(pp-pp(indx(1)))<1e-6)};
        otherwise
            warning('Wrong output argument number, no output set.')
    end

    
end

