%% Circuit Specifications 

Vin = 7.5;
Vout = -7.5; 
Iout = -.25;

fs=5e6;
omega = 2*pi*fs;
D=0.3; % duty cycle
type = 'v4h1-std'; % first harmonic solutions

% parameters

VdON = 0.7;                    
VbON = 0.7;                    
QLinv = 150;
QM = 150;
QLrec = 150;
QCinv = inf;
QCrec = inf; 

RdsON = .05;
RbON = .05;
RdON = .05;
Rin = 0.1;
Rout = 0.1;

% Metavariables:
% (Vin, Vout, Iout) -> (Vinv, Vrec, MIrec) conversion 
Vinv = Vin-Vout;  % Vin = Vinv - Vrec;
Vrec = -Vout;     % Vout = - Vrec;
% MIrec = MIinv-Iout; need to be estimated since MIinv is unknown

kI = 0.6;  
kR = Vrec/Vinv; % targeting Lrec=0

% QI and QR directly from kI, kR 
QI = (1-kI)/((1 - kI*Vrec/Vinv)/QLinv-kI*(1-Vrec/Vinv)/QM);
QR = (1-kR)/((1-kR*Vinv/Vrec)/QLrec-kR*(1-Vinv/Vrec)/QM);

vdON = VdON/Vrec;                    
vbON = VbON/Vinv;                    

% Average currents (real and normalized)
% <Iinv(t)> = MIinv;  <-Irec(t)> = MIrec
% <iinv(q)> = Miinv;  <-irec(q)> = 1 
% The value of MIinv is required for denormalization, but it is unknown now
% estimating it to a reasonabel value now; then, loop for getting the correct value 
Miinv = 1.2;  % rough estimation 
% since Iout = MIinv - MIrec = MIrec*(Vrec*Miinv/Vinv -1) we have:
MIrec = Iout/(Vrec*Miinv/Vinv -1);

gdsON = Vinv*Vinv/Vrec/MIrec/RdsON;
gbON = Vinv*Vinv/Vrec/MIrec/RbON;
gdON = Vrec/MIrec/RdON;     
gINV = 1/( Vrec*MIrec*Rin/Vinv^2-(Vinv-Vrec)*MIrec*Rout/Vinv^2 );
gCM = 1/( MIrec/Vinv*Rout );
gREC = 1/( (Vinv-Vrec)*MIrec/Vinv/Vrec*Rout);

%% Normalized solution from database 

addpath('..');
FILEoptimal = sprintf('../DCoptimal-%s-0%d.csv', type, round(10*D));
DBoptimal = dlmread(FILEoptimal,',',1,0);

% parameters extraction from Database
[theline, dist] = line_extract_nearest_v4b(DBoptimal,[kI,kR]);

if theline(17) > 1e-3 
    warning('the point is not existing in the DB, or it was not possible to find a solution.')
end

kI = theline(1);
kR = theline(2);
qI = theline(3);
qR = theline(4);
qM = theline(5);

iinv0 = theline(6); % should be zero
irec0 = theline(7);
vka0 = theline(8);

% show waveforms before optimization
if (true)
    
    [~, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
        vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);
 

    T = 2*pi*(0:255)'/255;
    Y = zeros(256,4);
    for ii=1:256
        Y(ii,:) = fZ(T(ii));
    end
    plot([T;T+2*pi], [Y;Y])
    legend('i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)')
end

%% Optimization (ZVS + ZVDS) for the lossy system

% loop in QQ (may be useful to add gradually losses and help convergence)
for QQ = [100 10 5 2 1 1 1 1]
    fToMinimize = @(x5) norm(solver4b (0, x5(1), x5(2), D*2*pi, x5(3), x5(4), x5(5), kI, kR, ...
        vdON/QQ, vbON/QQ, QI*QQ, QR*QQ, QM*QQ, QCinv*QQ, QCrec*QQ, gINV*QQ, gdsON*QQ, gbON*QQ, gCM*QQ, gdON*QQ, gREC*QQ)-[0; x5(1); 0; x5(2); 2*pi; -2*pi]);

    x0 = [irec0, vka0, qI, qR, qM];

    [R, res] = fminsearch(fToMinimize, x0 ,optimset('TolFun',1e-5,'MaxFunEvals',25000));

    iinv0 = 0;
    irec0 = R(1);
    vka0 = R(2);

    qI = R(3);
    qR = R(4);
    qM = R(5);
        
    if res > 1e-3
        fprintf(1, 'high residual (%g) at QQ=%g\n',res, QQ);
    end
    
    %disp([qI, qR, qM, iinv0, irec0, vka0, res]);
    
    % now get the estimation of Miinv and so MIrec
    [R, fZ] =solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
                    vdON/QQ, vbON/QQ, QI*QQ, QR*QQ, QM*QQ, QCinv*QQ, QCrec*QQ, gINV*QQ, gdsON*QQ, gbON*QQ, gCM*QQ, gdON*QQ, gREC*QQ);

    T = 2*pi*(0:255)'/255;
    Y = zeros(256,4);
    for ii=1:256
        Y(ii,:) = fZ(T(ii));
    end
 
    Miinv = mean(Y(:,1));
    Mirec = -mean(Y(:,2)); % just for check... should be 1 if optimization not failed
    
    MIrec = Iout/(Vrec*Miinv/Vinv -1);
    
    fprintf(1, 'QQ=%g, res=%g, Miinv=%g, Mirec=%g\n', QQ, res, Miinv, Mirec);
    

    gdsON = Vinv*Vinv/Vrec/MIrec/RdsON;
    gbON = Vinv*Vinv/Vrec/MIrec/RbON;
    gdON = Vrec/MIrec/RdON;  
    
    gINV = 1/( Vrec*MIrec*Rin/Vinv^2-(Vinv-Vrec)*MIrec*Rout/Vinv^2 );
    gCM = 1/( MIrec/Vinv*Rout );
    gREC = 1/( (Vinv-Vrec)*MIrec/Vinv/Vrec*Rout);
  

end

% plot normalized waveform
[~, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
                    vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);

T = 2*pi*(0:255)'/255;
Y = zeros(256,4);

for ii=1:256
    Y(ii,:) = fZ(T(ii));
end

plot([T;T+2*pi], [Y;Y])
legend('i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)')

%% Denormalization (non-isolated, std coupling, buck)

% Primary parameters (circuit elemets)

Linv = qM*Vinv*(Vinv-kI*Vrec)/2/pi/fs/kI/MIrec/Vrec;
Lrec = qM*(Vrec-kR*Vinv)/2/pi/fs/kR/MIrec;
M = qM*Vinv/2/pi/fs/MIrec;
Cinv = MIrec*Vrec/2/pi/fs/qI/Vinv^2;
Crec = MIrec/2/pi/fs/qR/Vrec;

% Secondary parameters (parasitics) %check

RLinv = qM*Vinv*(Vinv-kI*Vrec)/kI/MIrec/Vrec/QLinv;
RM = qM*Vinv/MIrec/QM; 
RLrec = qM*(Vrec-kR*Vinv)/kR/MIrec/QLrec;
RCinv = qI*Vinv^2/MIrec/Vrec/QCinv; 
RCrec = qR*Vrec/MIrec/QCrec; 

%% Spice Directives 

% avoid zeros to prevent spice errors
fprintf(1, '(cut & paste to spice)\n');
fprintf(1, ".param Vin=%g, Vout=%g, fs=%g, T={1/fs}, D=%g\n", ...
            Vin, Vout, fs, D);
fprintf(1, ".param Cinv=%s, Crec=%s, M=%s, Linv=%s, Lrec=%s\n", ...
            num2sip(Cinv), num2sip(Crec), num2sip(M), num2sip(max(Linv,1e-15)), num2sip(max(Lrec,1e-15)));
fprintf(1, ".param VdON=%g, VbON=%g, RLinv=%g, RLrec=%g, RM=%g, RCinv=%g, RCrec=%g, RdsON=%g, RdON=%g, Rin=%d, Rout=%d\n", ...
            VdON, VbON, max(RLinv,1e-6), max(RLrec,1e-6), RM, max(RCinv,1e-6), max(RCrec,1e-6), RdsON, RdON, max(Rin,1e-6), max(Rout,1e-6));
