%% Circuit Specifications 

Vin=12;
Vout=5;
Iout=100e-3;
fs=5e6;
omega = 2*pi*fs;
D=0.3; % duty cycle
type = 'v4h1-inv'; % first harmonic solutions

k = 0.98;
turns=2; %np/ns

QM = 100;
QLp = 100;
QLs = 100;

QLinv = 100;
QLrec = 100;

VdON = 0.7;
VbON = 0.7;

QCinv = inf;
QCrec = inf; 

Rin = 0.25;
RdsON = .1;  
RbON = .1;
RdON = .1;     
Rout = .25; 

% Metavariables:
% (Vin, Vout, Iout) -> (Vinv, Vrec, MIrec) conversion 
Vinv = Vin;
Vrec = Vout; 
MIrec = Iout; % MIrec = <Irec(t)>

vdON = VdON / Vrec;                    
vbON = VbON / Vinv; 
gINV = Vinv^2/Vrec/MIrec/Rin;
gdsON = Vinv^2/Vrec/MIrec/RdsON;
gbON = Vinv^2/Vrec/MIrec/RbON;
gCM = inf;
gdON = Vrec/MIrec/RdON;
gREC = Vrec/MIrec/Rout;

kI=-0.8;
kR=-0.8;

% Computing QI & QM, need the values of M, Lp and Ls
qM = -rand; % no need for the exact value, everything is proportional to qM
M = -qM*Vinv/MIrec/omega;  
Lp = M/k*turns;
Ls = M/k/turns;
Linv = Vinv*(qM*Vinv/MIrec/omega)/Vrec/kI - Lp;            
Lrec = Vrec*(qM/MIrec/omega)/kR - Ls;   

QI = 1/(Linv/(Linv+Lp+Vinv/Vrec*M)/QLinv + ...
        Lp/(Linv+Lp+Vinv/Vrec*M)/QLp + ...
        M*Vinv/Vrec/(Linv+Lp+Vinv/Vrec*M)/QM);
QR = 1/(Lrec/(Lrec+Ls+Vrec/Vinv*M)/QLrec + ...
        Ls/(Lrec+Ls+Vrec/Vinv*M)/QLs + ...
        M*Vrec/Vinv/(Lrec+Ls+Vrec/Vinv*M)/QM);
clear('qM','M','Lp','Ls','Linv','Lrec');

%% Normalized solution from database 

addpath('..');
FILEoptimal = sprintf('../DCoptimal-%s-0%d.csv', type, round(10*D));
DBoptimal = dlmread(FILEoptimal,',',1,0);

% parameters extraction from Database
[theline, dist] = line_extract_nearest_v4b(DBoptimal,[kI,kR]);

if theline(17) > 1e-3
    warning('the point is not existing in the DB, or it was not possible to find a solution.')
end

kI = theline(1);
kR = theline(2);
qI = theline(3);
qR = theline(4);
qM = theline(5);

iinv0 = theline(6); % should be zero
irec0 = theline(7);
vka0 = theline(8);

% show waveforms before optimization
if (false)
    
    [~, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
        vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);
 

    T = 2*pi*(0:255)'/255;
    Y = zeros(256,4);
    for ii=1:256
        Y(ii,:) = fZ(T(ii));
    end
    plot([T;T+2*pi], [Y;Y])
    legend('i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)')
end

%% Optimization (ZVS + ZVDS) for the lossy system

% loop in QQ (may be useful to add gradually losses and help convergence)
for QQ = [10 2 1]
    fToMinimize = @(x5) norm(solver4b (0, x5(1), x5(2), D*2*pi, x5(3), x5(4), x5(5), kI, kR, ...
        vdON/QQ, vbON/QQ, QI*QQ, QR*QQ, QM*QQ, QCinv*QQ, QCrec*QQ, gINV*QQ, gdsON*QQ, gbON*QQ, gCM*QQ, gdON*QQ, gREC*QQ)-[0; x5(1); 0; x5(2); 2*pi; -2*pi]);

    x0 = [irec0, vka0, qI, qR, qM];

    [R, res] = fminsearch(fToMinimize, x0 ,optimset('TolFun',1e-5,'MaxFunEvals',25000));

    iinv0 = 0;
    irec0 = R(1);
    vka0 = R(2);

    qI = R(3);
    qR = R(4);
    qM = R(5);
        
    if res > 1e-3
        fprintf(1, 'high residual at QQ=%g\n',QQ);
    end
    % disp([qI, qR, qM, iinv0, irec0, vka0, res]);
end

% plot normalized waveform
[~, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
                    vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);

T = 2*pi*(0:255)'/255;
Y = zeros(256,4);

for ii=1:256
    Y(ii,:) = fZ(T(ii));
end

plot([T;T+2*pi], [Y;Y])
legend('i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)')
     
%% Denormalization (isolated, inv coupling)

% Primary parameters (circuit elemets)
Cinv = Vrec*MIrec/Vinv^2/omega/qI;
Crec = MIrec/Vrec/omega/qR;

M = -qM*Vinv/MIrec/omega;
Lp = M/k*turns;
Ls = M/k/turns;
Linv = Vinv*(qM*Vinv/MIrec/omega)/Vrec/kI - Lp;            
Lrec = Vrec*(qM/MIrec/omega)/kR - Ls;   

% Secondary parameters (parasitics)
RM = omega*M/QM;
RLs = omega*Ls/QLs;
RLp = omega*Lp/QLp;
RLinv = omega*Linv/QLinv;
RLrec = omega*Lrec/QLrec;
RCinv = 1/omega/Cinv/QCinv;
RCrec = 1/omega/Crec/QCrec;

%% Spice Directives 

% avoid zeros to prevent spice errors
fprintf(1, '(cut & paste to spice)\n');
fprintf(1, ".param Vin=%g, Vout=%g, fs=%g, T={1/fs}, D=%g, n=%g\n", ...
            Vinv, Vrec, fs, D, turns);
fprintf(1, ".param Cinv=%s, Crec=%s, M=%s, Linv=%s, Lrec=%s, Lp=%s, Ls=%s\n", ...
            num2sip(Cinv), num2sip(Crec), num2sip(M), num2sip(max(Linv,1e-12)), num2sip(max(Lrec,1e-12)), num2sip(Lp), num2sip(Ls));
fprintf(1, ".param VdON=%g, VbON=%g, RLinv=%g, RLrec=%g, RLp=%g, RLs=%g, RM=%g, RCinv=%g, RCrec=%g\n", ...
            VdON, VbON, max(RLinv,1e-6), max(RLrec,1e-6), RLp, RLs, RM, max(RCinv,1e-6), max(RCrec,1e-6));
fprintf(1, ".param RdsON=%g, RdON=%g, RbON=%g, Rin=%d, Rout=%d\n", ...
            RdsON, RdON, RbON, Rin, Rout);