%% Circuit Specifications 

D=0.3; % duty cycle
type = 'v4h1-std'; % first harmonic solutions

QM = 100;
QI = 100;
QR = 100;
QCinv = inf;
QCrec = inf; 

vdON = 0.1;                    
vbON = 0.1; 
gINV = 500;
gdsON = 250;
gbON = 250;
gCM = inf;
gdON = 300;
gREC = 500;

kI=0.8;
kR=0.8;



%% Load normalized solution from database 

addpath('..');
FILEoptimal = sprintf('../DCoptimal-%s-0%d.csv', type, round(10*D));
DBoptimal = dlmread(FILEoptimal,',',1,0);

% parameters extraction from Database
[theline, dist] = line_extract_nearest_v4b(DBoptimal,[kI,kR]);

if theline(17) > 1e-3 
    warning('the point is not existing in the DB, or it was not possible to find a solution.')
end

kI = theline(1);
kR = theline(2);
qI = theline(3);
qR = theline(4);
qM = theline(5);

iinv0 = theline(6); % should be zero
irec0 = theline(7);
vka0 = theline(8);

% show waveforms before optimization
if (false)
    
[~, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
    vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);
 

    T = 2*pi*(0:255)'/255;
    Y = zeros(256,4);
    for ii=1:256
        Y(ii,:) = fZ(T(ii));
    end
    plot([T;T+2*pi], [Y;Y])
    legend('i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)')
end

%% Optimization (ZVS + ZVDS) for the lossy system

% loop in QQ (may be useful to add gradually losses and help convergence)
for QQ = [10 2 1]
    fToMinimize = @(x5) norm(solver4b (0, x5(1), x5(2), D*2*pi, x5(3), x5(4), x5(5), kI, kR, ...
        vdON/QQ, vbON/QQ, QI*QQ, QR*QQ, QM*QQ, QCinv*QQ, QCrec*QQ, gINV*QQ, gdsON*QQ, gbON*QQ, gCM*QQ, gdON*QQ, gREC*QQ)-[0; x5(1); 0; x5(2); 2*pi; -2*pi]);

    x0 = [irec0, vka0, qI, qR, qM];

    [R, res] = fminsearch(fToMinimize, x0 ,optimset('TolFun',1e-5,'MaxFunEvals',25000));

    iinv0 = 0;
    irec0 = R(1);
    vka0 = R(2);

    qI = R(3);
    qR = R(4);
    qM = R(5);
        
    if res > 1e-3
        fprintf(1, 'high residual at QQ=%g\n',QQ);
    end
    % disp([qI, qR, qM, iinv0, irec0, vka0, res]);
end

% plot normalized waveform
[~, fZ] = solver4b (iinv0, irec0, vka0, D*2*pi, qI, qR, qM, kI, kR, ...
                    vdON, vbON, QI, QR, QM, QCinv, QCrec, gINV, gdsON, gbON, gCM, gdON, gREC);

T = 2*pi*(0:255)'/255;
Y = zeros(256,4);

for ii=1:256
    Y(ii,:) = fZ(T(ii));
end

plot([T;T+2*pi], [Y;Y])
legend('i_{inv}(\theta)','i_{rec}(\theta)','v_{DS}(\theta)','v_{KA}(\theta)')
        
%% Spice Directives 

% avoid infinities to prevent spice errors
fprintf(1, '(cut & paste to spice)\n');
fprintf(1, '.param qI=%g, qR=%g, qM=%g, kI=%g, kR=%g, theta2=2*pi*%g\n', ...
            qI, qR, qM, kI, kR, D);
fprintf(1, ".param vdON=%g, vbON=%g, QQI=%g, QQM=%g, QQR=%g, QQCinv=%g, QQCrec=%g, gdsON=%g, gbON=%g, gdON=%g, gINV=%g, gCM=%g, gREC=%g\n", ...
            vdON, vbON, min(QI,1e6), min(QM,1e6), min(QR,1e6), min(QCinv,1e6), min(QCrec,1e6), min(gdsON,1e6), min(gbON,1e6), min(gdON,1e6), min(gINV,1e6), min(gCM,1e6), min(gREC,1e6));
