function fullline = line_computeresiduals_v4b(shortline, theta2)
% computes 2pi values (iinv2pi, irec2pi, vds2pi, vka2pi, iif), angles (theta1, theta2, theta3), and full (res5) and relaxed (res4) residuals

    if size(shortline,1) ~= 1 || size(shortline,2) ~= 8
        error('Error in shortline size');
    end

    if size(theta2,1) ~= 1 || size(theta2,2) ~= 1
        error('Error in theta2 size');
    end

    kI = shortline(1);
    kR = shortline(2);
    qI = shortline(3);
    qR = shortline(4);
    qM = shortline(5);
    
    iinv0 = shortline(6);
    irec0 = shortline(7);
    vka0 = shortline(8);

    % shortline is: 'kI', 'kR', 'qI', 'qR', 'qM', 'iinv0', 'irec0'
    % line is: 'kI', 'kR', 'qI', 'qR', 'qM', 'iinv0', 'irec0', 'iinv2pi', 'irec2pi', 'ZVDS', 'ZVS', 'iiF', 'theta1', 'theta2', 'theta3', 'res5', 'res4'
    % solver3 is: 'iinv2pi', 'irec2pi', 'ZVDS', 'ZVS', 'iiF'

    warning('OFF', 'MATLAB:singularMatrix');
    warning('OFF', 'MATLAB:nearlySingularMatrix');

    try
        vv = solver4b (iinv0, irec0, vka0, theta2, qI, qR, qM, kI, kR, ...
            0, 0, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf);

        % No residuals larger than 1 are accepted
        res5 = min(norm(vv-[0; irec0; 0; vka0; 2*pi; -2*pi]), 1);
        res4 = min(norm(vv-[iinv0; irec0; 0; vka0; 2*pi; -2*pi]), 1);

% headers = {'kI', 'kR', 'qI', 'qR', 'qM', 'iinv0', 'irec0', 'vka0', ... % 1 to 8 [8]
%          'iinv2pi[ZVDS]', 'irec2pi', 'vds2pi[ZVS]', 'vka2pi', 'lasttbon', 'iiF', ... % 9 to 14 [6] 
%          'theta2', 'deprecated', ... % 15 to 16 [2] 
%          'resoptimal', 'resrelaxed', 'lastZVDSreduction', ... % 17 to 19 [3]
%          'spiceiinv', 'spiceirec', 'spicevds', 'spicevka', 'spiceall', ... % 20 to 24 [5]
%          'reserved', 'reserved', 'reserved', 'reserved', 'reserved', 'reserved', 'reserved', 'reserved'}; % 25 to 32 [8]
%  
% 'kI', 'kR', 'qI', 'qR', 'qM', 'iinv0', 'irec0', 'vka0' are self-explaining
% 'iinv2pi[ZVDS]', 'irec2pi', 'vds2pi[ZVS]', 'vka2pi', 'lasttbon', 'iiF' still self-explaining
%   iinv(2*pi)=ZVDS is the value of iinv at the MOS turn on instant (should be zero, or minimized in module in relaxed mode)
%   vds(2*pi)=ZVS is the value of vCinv/vds at the MOS turn on instant (should be zero) 
%   lasttbon the last time when body diode turned on (2*pi not turned on)
%   iiF is the value of Integrate[irec[t],t,0,2*pi] (should be -2*pi)
% theta2 is the normalized time when the MOS turn OFF
% resoptimal is the residual value using the optimal design (ZVS + ZVDS)
% resrelaxed is the residual value using the relaxed design (ZVS only + optional current minimization)
% lastZVDSreduction is the last reduction for which convegence was not reached [0.1, 0.01, or 0.001]
% spiceiinv, spiceirec, spicevds, spicevka are the (normalized) integral of the differences between spice and matlab waveforms
% unused (or uncomputed) fields are set to -1.
        
        if any(isnan(vv)) || any(isinf(vv))
            fullline = [shortline, [0, 0, 0, 0, 0, 0], [theta2, -1], [1, 1, -1] -ones(1,13)];
        else
            fullline = [shortline, vv', [theta2, -1], [res5, res4, -1], -ones(1,13)];
        end
        
    catch 
        % Qualche errore  avvenuto
        warning('Problem using solver4() function at kI=%g, kR=%g, theta2=%g*2*pi. Assigning residual values of 1.', kI, kR, theta2/2/pi);

        fullline = [shortline, [0, 0, 0, 0, 0], [0, theta2, 0], 1, 1, -ones(1,14)];
    end

    warning('ON', 'MATLAB:singularMatrix');
    warning('ON', 'MATLAB:nearlySingularMatrix');

end