function varargout = line_refine_v4bshort(inshortline, target, theta2, varargin)
% Refine a line starting from a starting point
%
% inshortline is the input line (to be used as starting point) - row vector
% target is the desired [irec(2*pi) vcrec(2*pi) lasttbon iif] - column vector
% theta2 is theta2 to be passed to solver3 - scalar
% shortline is [kI, kR, qI, qR, qM, iinv0, irec0, vka0]

    if size(inshortline,1) ~= 1 || size(inshortline,2) ~= 8
        error('Error in input line size');
    end

    if size(target,1) ~= 4 || size(target,2) ~= 1
        error('Error in target size');
    end

    if size(theta2,1) ~= 1 || size(theta2,2) ~= 1
        error('Error in theta2 size');
    end
    
    % minimization to be run with 5 (any iinv0) or 6 (tagret iinv0) arguments
    % x0 = [irec0, vka0, qI, qR, qM] or [irec0, vka0, qI, qR, qM, iinv0]
    % solver4 (iinv0, irec0, vka0, theta2, qI, qR, qM, kI, kR, ... );
    % solver4 (iinv0|x0(6), x0(1), x0(2), theta2, x0(3), x0(4), x0(5), kI, kR, ... );
    % 
    % distance from another line:
    % shortline = 'kI', 'kR', 'qI', 'qR', 'qM', 'iinv0', 'irec0', 'vka0'
    % distance is x0 - shortline([7, 8, 3:5]) or x0 - shortline([7, 8, 3:6])

    kI = inshortline(1);
    kR = inshortline(2);

    fRemove5 = @(x) 0;
    fRemove6 = @(x) 0;

    % Handling of unwanted solutions -- work in progress
    switch nargin
        case 3
%            fRemove5 = @(x) 0;
%            fRemove6 = @(x) 0;
        case 5
            % varargin{1} has to be a shortline, varargin{2} the minimum distance
            avoid_db = varargin{1};
            avoid_dist = varargin{2};
            if size(avoid_db,2) < 8
                error('Error in size of avoid-result db');
            end
            if size(avoid_dist,1) ~= 1 || size(avoid_dist,2) ~= 1
                error('Error in size of avoid-result coefficient');
            end
            % step 1: look into avoid db if some cases are applicable
            pp = (avoid_db(:,1)-kI).^2+(avoid_db(:,2)-kR).^2;
            toavoid = avoid_db(pp<1e-5,:);
            
%            % works only for ZVS and ZVDS for now -- avoidshortline(6) = iinv(0) is not used
%            z0 = [avoidshortline(7), avoidshortline(8), avoidshortline(3), avoidshortline(4), avoidshortline(5)];
%            fRemove = @(x) (dist/norm(x-z0)-1)*(norm(x-z0)<dist);
%            fRemove5 = @(x5) avoid_dist/norm(x5 - avoid_db([7, 8, 3:5]))^2;
%            fRemove6 = @(x6) avoid_dist/norm(x6 - avoid_db([7, 8, 3:6]))^2;
            fRemove = @(d,a) (a^2/d^2-1)*(d<a);
            %fRemove5 = @(x5) fRemove(norm(x5 - avoid_db([7, 8, 3:5])), avoid_dist);
            %fRemove6 = @(x6) fRemove(norm(x6 - avoid_db([7, 8, 3:6])), avoid_dist);

            for jj=1:size(toavoid,1)
%                fRemove5 = @(x5) fRemove5(x5) + fRemove(norm(x5 - toavoid(jj,[7, 8, 3:5])), avoid_dist);
%                fRemove6 = @(x6) fRemove6(x6) + fRemove(norm(x6 - toavoid(jj,[7, 8, 3:6])), avoid_dist);
                % using only qI qR qM, and normalizing
                fRemove5 = @(x5) fRemove5(x5) + fRemove(norm(1 - x5(3:5)./toavoid(jj,3:5)), avoid_dist);
                fRemove6 = @(x6) fRemove6(x6) + fRemove(norm(1 - x6(3:5)./toavoid(jj,3:5)), avoid_dist);
            end
        otherwise
            error('Wrong number of argument')
    end
    
    % do not run if kI, kR=0 or out of existence condition kI*kR >= 1
    if abs(kI)<1e-5 || abs(kR)<1e-5 || kI*kR >= 1
        outshortline = inshortline;
        residual = 1;

    else
        % Here a solution may exist
        
        % Target is either [iinv2pi(ZVDS); vds2pi(ZVS); 2*pi; -2*pi]
        % or [NaN; vds2pi(ZVS); 2*pi; -2*pi]
        % prepare to run with 5 or 6 constraints
        % x0 = [irec0, vka0, qI, qR, qM] or [irec0, vka0, qI, qR, qM, iinv0]
            % solver4 (iinv0, irec0, vka0, theta2, qI, qR, qM, kI, kR, ... );
        if isnan(target(1))
%            % any iinv2pi(ZVDS) is OK
%            fToMinimize = @(x6) norm(solver4b (x6(6), x6(1), x6(2), theta2, x6(3), x6(4), x6(5), kI, kR, ...
%                0, 0, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf)-[x6(6); x6(1); target(2); x6(2); target(3:4)]) + ...
%                fRemove6(x6);
%            x0 = [inshortline(7), inshortline(8), inshortline(3), inshortline(4), inshortline(5), inshortline(6)];
            % new approach: minimize residual + iinv0
            finv0res4 = @(iinv0, res4) abs(iinv0) + 1e2*(1e3*res4)^6;
            fToMinimize = @(x6) finv0res4(x6(6), norm(solver4b (x6(6), x6(1), x6(2), theta2, x6(3), x6(4), x6(5), kI, kR, ...
                0, 0, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf)-[x6(6); x6(1); target(2); x6(2); target(3:4)])) + ...
                fRemove6(x6);
            x0 = [inshortline(7), inshortline(8), inshortline(3), inshortline(4), inshortline(5), inshortline(6)];
        else
            % asking for the desired iinv2pi(ZVDS)
            fToMinimize = @(x5) norm(solver4b (target(1), x5(1), x5(2), theta2, x5(3), x5(4), x5(5), kI, kR, ...
                0, 0, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf, inf)-[target(1); x5(1); target(2); x5(2); target(3:4)]) + ...
                fRemove5(x5);
            x0 = [inshortline(7), inshortline(8), inshortline(3), inshortline(4), inshortline(5)];
        end

        % shortline is: 'kI', 'kR', 'qI', 'qR', 'qM', 'iinv0', 'irec0', 'vka0'
        % solver4 is: 'iinv2pi(ZVDS)', 'irec2pi', 'vds2pi(VDS)', 'vka2pi', 'iiF'

        warning('OFF', 'MATLAB:singularMatrix');
        warning('OFF', 'MATLAB:nearlySingularMatrix');

        try
            %tic;
            [vv, residual] = fminsearch(fToMinimize, x0 ,optimset('TolFun',1e-9,'MaxFunEvals',2500));       
            %toc;

            if isnan(target(1))
                outshortline = [kI, kR, vv(3:5), vv(6), vv(1:2)];
            else
                outshortline = [kI, kR, vv(3:5), target(1), vv(1:2)];
            end

        catch 
            % Qualche errore  avvenuto
            warning('Problem using minimization function at kI=%g, kR=%g, theta2=%g*2*pi.', kI, kR, theta2/2/pi);
            outshortline = inshortline;
        end

        warning('ON', 'MATLAB:singularMatrix');
        warning('ON', 'MATLAB:nearlySingularMatrix');
    
    end
    
    switch nargout
        case {0, 1}
            varargout = {outshortline};
        case 2
            varargout = {outshortline, residual};
        otherwise
            warning('Wrong output argument number, no output set.')
    end

   
end